/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.util.io;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 18.08.2006
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: LazyFileInputStream.java 12843 2007-02-16 09:11:18Z peterit $
 */
public class LazyFileInputStream extends CloneableInputStream {

  protected final File file;
  protected FileInputStream dataStream; 
  
  public LazyFileInputStream(File file) {
    this.file = file;
  }
  
  /**
   * Return a file read by this stream.
   * Optional feature of LazyFileInputStream class (and its descendants). 
   * A file returned may be used for NIO operations. 
   */
  protected File getSourceFile() {
    return file;
  }
  
  /**
   * For use in NIO operations.
   * 
   * @return - this stream file channel
   */
  public FileChannel getChannel() throws IOException {
    return stream().getChannel();
  }
  
  /**
   * A file will be readed on demand
   */
  protected FileInputStream stream() throws IOException {
    if (dataStream == null) {
      dataStream = new FileInputStream(file);
    }
    return dataStream;
  }

  @Override
  public void close() throws IOException {
    if (dataStream != null)
      super.close();
    // else nothing to close
  }

  @Override
  public LazyFileInputStream clone() {
    return new LazyFileInputStream(file);
  }

  @Override
  public boolean equals(Object obj) {
    if (obj instanceof LazyFileInputStream)
      return ((LazyFileInputStream) obj).file.equals(file);
    return super.equals(obj);
  }

  @Override
  public String toString() {
    return super.toString() + ", [" + file.getAbsolutePath() + "]";
  }

  @Override
  protected void finalize() throws Throwable {
    try {
      close();
    } catch(IOException e) {
      // not matter 
    }
    super.finalize();
  }
  
}
