/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.util.io;

import java.io.IOException;
import java.io.InputStream;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 17.08.2006
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: LazyInputStream.java 12843 2007-02-16 09:11:18Z peterit $
 */
public abstract class LazyInputStream extends InputStream {
  
  protected abstract InputStream stream() throws IllegalStateException, IOException;
  
  /* (non-Javadoc)
   * @see java.io.InputStream#read()
   */
  @Override
  public int read() throws IOException {
    try {
      return stream().read();
    } catch(IllegalStateException e) {
      throw new IOException("IllegalStateException: " + e.getMessage());
    }
  }

  /* (non-Javadoc)
   * @see java.io.InputStream#available()
   */
  @Override
  public int available() throws IOException {
    try {
      return stream().available();
    } catch(IllegalStateException e) {
      throw new IOException("IllegalStateException: " + e.getMessage());
    }
  }

  /* (non-Javadoc)
   * @see java.io.InputStream#close()
   */
  @Override
  public void close() throws IOException {
    try {
      stream().close();
    } catch(IllegalStateException e) {
      throw new IOException("IllegalStateException: " + e.getMessage());
    }
  }

  /* (non-Javadoc)
   * @see java.io.InputStream#mark(int)
   */
  @Override
  public synchronized void mark(int readlimit) {
    try {
      stream().mark(readlimit);
    } catch(IOException e) {
      throw new RuntimeException("IOException: " + e.getMessage(), e);      
    } catch(IllegalStateException e) {
      throw new RuntimeException("IllegalStateException: " + e.getMessage(), e);
    }
  }

  /* (non-Javadoc)
   * @see java.io.InputStream#markSupported()
   */
  @Override
  public boolean markSupported() {
    try {
      return stream().markSupported();
    } catch(IOException e) {
      throw new RuntimeException("IOException: " + e.getMessage(), e);      
    } catch(IllegalStateException e) {
      throw new RuntimeException("IllegalStateException: " + e.getMessage(), e);
    }
  }

  /* (non-Javadoc)
   * @see java.io.InputStream#read(byte[], int, int)
   */
  @Override
  public int read(byte[] b, int off, int len) throws IOException {
    try {
      return stream().read(b, off, len);
    } catch(IllegalStateException e) {
      throw new IOException("IllegalStateException: " + e.getMessage());
    }
  }

  /* (non-Javadoc)
   * @see java.io.InputStream#read(byte[])
   */
  @Override
  public int read(byte[] b) throws IOException {
    try {
      return stream().read(b);
    } catch(IllegalStateException e) {
      throw new IOException("IllegalStateException: " + e.getMessage());
    }
  }

  /* (non-Javadoc)
   * @see java.io.InputStream#reset()
   */
  @Override
  public synchronized void reset() throws IOException {
    try {
      stream().reset();
    } catch(IllegalStateException e) {
      throw new IOException("IllegalStateException: " + e.getMessage());
    }
  }

  /* (non-Javadoc)
   * @see java.io.InputStream#skip(long)
   */
  @Override
  public long skip(long n) throws IOException {
    try {
      return stream().skip(n);
    } catch(IllegalStateException e) {
      throw new IOException("IllegalStateException: " + e.getMessage());
    }
  }
  
}
